# Generated by Django 4.2.16 on 2024-10-07 12:10
import logging

from django.db import migrations

from core.utils import insert_role_right_for_system, remove_role_right_for_system

logger = logging.getLogger(__name__)

def add_rights(apps, schema_editor):
    insert_role_right_for_system(64, 101500, apps)
    insert_role_right_for_system(64, 154901, apps)

def remove_rights(apps, schema_editor):
    remove_role_right_for_system(64, 101500, apps)
    remove_role_right_for_system(64, 154901, apps)

class Migration(migrations.Migration):

    dependencies = [
        ('contract', '0021_alter_contract_date_created_and_more'),
    ]

    operations = [
        migrations.RunPython(add_rights, reverse_code=remove_rights),
    ]
