# Generated by Django 3.0.3 on 2020-12-30 10:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('policyholder', '0004_auto_20201214_1302'),
        ('contract', '0007_auto_20201228_1310'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contract',
            name='policy_holder',
            field=models.ForeignKey(blank=True, db_column='PolicyHolderUUID', null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='policyholder.PolicyHolder'),
        ),
    ]
