# Generated by Django 3.0.3 on 2020-12-28 13:10

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('policy', '0001_initial'),
        ('contract', '0006_auto_20201228_1218'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contractcontributionplandetails',
            name='policy',
            field=models.ForeignKey(db_column='PolicyID', on_delete=django.db.models.deletion.DO_NOTHING, to='policy.Policy'),
        ),
        migrations.AlterField(
            model_name='historicalcontractcontributionplandetails',
            name='policy',
            field=models.ForeignKey(blank=True, db_column='PolicyID', db_constraint=False, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='policy.Policy'),
        ),
    ]
