# Generated by Django 3.0.3 on 2020-12-22 10:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contract', '0002_auto_20201221_1547'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contract',
            name='code',
            field=models.CharField(db_column='Code', max_length=64),
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='code',
            field=models.CharField(db_column='Code', max_length=64),
        ),
    ]
