# Generated by Django 4.2.16 on 2024-10-30 08:31

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ("policy", "0010_merge_20240912_1248"),
        ("contract", "0022_add_admin_rights"),
    ]

    operations = [
        migrations.AddField(
            model_name="contractcontributionplandetails",
            name="amount",
            field=models.DecimalField(
                blank=True,
                db_column="Amount",
                decimal_places=2,
                max_digits=18,
                null=True,
            ),
        ),
        migrations.AddField(
            model_name="historicalcontractcontributionplandetails",
            name="amount",
            field=models.DecimalField(
                blank=True,
                db_column="Amount",
                decimal_places=2,
                max_digits=18,
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="contractcontributionplandetails",
            name="policy",
            field=models.ForeignKey(
                blank=True,
                db_column="PolicyID",
                null=True,
                on_delete=django.db.models.deletion.DO_NOTHING,
                to="policy.policy",
            ),
        ),
    ]
