# Generated by Django 3.0.3 on 2021-01-19 12:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contract', '0009_auto_20210118_1427'),
    ]

    operations = [
        migrations.AlterField(
            model_name='contractcontributionplandetails',
            name='contract_details',
            field=models.ForeignKey(db_column='ContractDetailsUUID', on_delete=django.db.models.deletion.CASCADE, to='contract.ContractDetails'),
        ),
        migrations.AlterField(
            model_name='contractdetails',
            name='contract',
            field=models.ForeignKey(db_column='ContractUUID', on_delete=django.db.models.deletion.CASCADE, to='contract.Contract'),
        ),
    ]
