# Generated by Django 3.2.16 on 2023-01-26 09:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contract', '0019_amend_perms'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='historicalcontract',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical contract', 'verbose_name_plural': 'historical contracts'},
        ),
        migrations.AlterModelOptions(
            name='historicalcontractcontributionplandetails',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical contract contribution plan details', 'verbose_name_plural': 'historical contract contribution plan detailss'},
        ),
        migrations.AlterModelOptions(
            name='historicalcontractdetails',
            options={'get_latest_by': ('history_date', 'history_id'), 'ordering': ('-history_date', '-history_id'), 'verbose_name': 'historical contract details', 'verbose_name_plural': 'historical contract detailss'},
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalcontractcontributionplandetails',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
        migrations.AlterField(
            model_name='historicalcontractdetails',
            name='history_date',
            field=models.DateTimeField(db_index=True),
        ),
    ]
