# Generated by Django 3.0.3 on 2020-12-21 15:47

import core.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('contract', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='contract',
            name='amendment',
            field=models.IntegerField(blank=True, db_column='Amendment', null=True),
        ),
        migrations.AddField(
            model_name='contract',
            name='code',
            field=models.CharField(db_column='Code', default='CODE1', max_length=255),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='historicalcontract',
            name='amendment',
            field=models.IntegerField(blank=True, db_column='Amendment', null=True),
        ),
        migrations.AddField(
            model_name='historicalcontract',
            name='code',
            field=models.CharField(db_column='Code', default='CODE2', max_length=255),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='contract',
            name='amount_due',
            field=models.FloatField(blank=True, db_column='AmountDue', null=True),
        ),
        migrations.AlterField(
            model_name='contract',
            name='amount_notified',
            field=models.FloatField(blank=True, db_column='AmountNotified', null=True),
        ),
        migrations.AlterField(
            model_name='contract',
            name='amount_rectified',
            field=models.FloatField(blank=True, db_column='AmountRectified', null=True),
        ),
        migrations.AlterField(
            model_name='contract',
            name='date_approved',
            field=core.fields.DateTimeField(blank=True, db_column='DateApproved', null=True),
        ),
        migrations.AlterField(
            model_name='contract',
            name='date_payment_due',
            field=core.fields.DateField(blank=True, db_column='DatePaymentDue', null=True),
        ),
        migrations.AlterField(
            model_name='contract',
            name='payment_reference',
            field=models.CharField(blank=True, db_column='PaymentReference', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='contract',
            name='state',
            field=models.SmallIntegerField(blank=True, db_column='State', null=True),
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='amount_due',
            field=models.FloatField(blank=True, db_column='AmountDue', null=True),
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='amount_notified',
            field=models.FloatField(blank=True, db_column='AmountNotified', null=True),
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='amount_rectified',
            field=models.FloatField(blank=True, db_column='AmountRectified', null=True),
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='date_approved',
            field=core.fields.DateTimeField(blank=True, db_column='DateApproved', null=True),
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='date_payment_due',
            field=core.fields.DateField(blank=True, db_column='DatePaymentDue', null=True),
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='payment_reference',
            field=models.CharField(blank=True, db_column='PaymentReference', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='historicalcontract',
            name='state',
            field=models.SmallIntegerField(blank=True, db_column='State', null=True),
        ),
    ]
